'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _updateFunctionCall = require('./update-function-call');

var _updateFunctionCall2 = _interopRequireDefault(_updateFunctionCall);

var _updateChain = require('./update-chain');

var _updateChain2 = _interopRequireDefault(_updateChain);

var _getPropertyName = require('./get-property-name');

var _getPropertyName2 = _interopRequireDefault(_getPropertyName);

var _buildCall = require('./build-call');

var _buildCall2 = _interopRequireDefault(_buildCall);

var _updatePartial = require('./update-partial');

var _updatePartial2 = _interopRequireDefault(_updatePartial);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

exports.default = function () {
  return {
    visitor: {
      ArrowFunctionExpression: {
        exit: function exit(path) {
          var _path$node = path.node;
          var expression = _path$node.expression;
          var params = _path$node.params;
          var body = _path$node.body;

          if (!expression) return;

          (0, _updatePartial2.default)(path, body, params);
        }
      },
      FunctionExpression: {
        exit: function exit(path) {
          var _path$node2 = path.node;
          var params = _path$node2.params;
          var fnStatements = _path$node2.body.body;

          if (fnStatements.length !== 1) return;

          var returnStatement = fnStatements[0];
          if (!t.isReturnStatement(returnStatement)) return;

          var argument = returnStatement.argument;

          if (!argument) return;

          (0, _updatePartial2.default)(path, argument, params);
        }
      },
      CallExpression: function CallExpression(path) {
        var _path$node3 = path.node;
        var replaced = _path$node3.replaced;
        var callee = _path$node3.callee;

        if (replaced) return;

        if (t.isIdentifier(callee) && callee.name === '_') {
          (0, _updateChain2.default)(path);
        } else if (t.isMemberExpression(callee)) {
          var object = callee.object;

          if (!t.isIdentifier(object) || object.name !== '_') return;

          (0, _getPropertyName2.default)(callee) === 'chain' ? (0, _updateChain2.default)(path) : (0, _updateFunctionCall2.default)(path);
        }
      }
    }
  };
};